package gov.usgs.earthquake.nshmp.www;

import java.nio.file.Path;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.logging.Logger;

import org.crac.Context;
import org.crac.Resource;

import com.amazonaws.serverless.proxy.internal.testutils.MockLambdaContext;
import com.amazonaws.serverless.proxy.model.AwsProxyRequest;
import com.google.common.util.concurrent.MoreExecutors;

import io.micronaut.context.annotation.Value;
import io.micronaut.crac.OrderedResource;
import io.micronaut.function.aws.proxy.MicronautLambdaHandler;
import jakarta.inject.Singleton;

/**
 * Prime application for AWS Lambda snapstart.
 */
@Singleton
public class PrimingResource implements OrderedResource {

  @Value("${nshmp-haz.model-path}")
  private Path modelPath;

  @Value("${nshmp-haz.context-path}")
  String contextPath;

  @Override
  public void beforeCheckpoint(Context<? extends Resource> context) throws Exception {
    Logger log = Logger.getAnonymousLogger();
    log.info("Before checkpoint");
    ServletUtil.model(ServletUtil.loadModel(modelPath));

    Optional<String> location = Optional.ofNullable(System.getenv("TEST_LOCATION"));

    try (MicronautLambdaHandler handler = new MicronautLambdaHandler()) {
      handler.handleRequest(
          getAwsProxyRequest(contextPath + "/hazard"),
          new MockLambdaContext());

      // log.info("Test location");
      // if (location.isPresent()) {
      // log.info("Location: " + location.get());
      // handler.handleRequest(
      // getAwsProxyRequest(contextPath + "/hazard" + location.get()),
      // new MockLambdaContext());
      // }

      // handler.handleRequest(
      // getAwsProxyRequest(contextPath + "/probability"),
      // new MockLambdaContext());

      // handler.handleRequest(
      // getAwsProxyRequest(contextPath + "/rate"),
      // new MockLambdaContext());

      // handler.handleRequest(
      // getAwsProxyRequest(contextPath + "/disagg"),
      // new MockLambdaContext());

      // handler.handleRequest(
      // getAwsProxyRequest(contextPath + "/map"),
      // new MockLambdaContext());

      // handler.handleRequest(
      // getAwsProxyRequest(contextPath),
      // new MockLambdaContext());
    }
  }

  @Override
  public void afterRestore(Context<? extends Resource> context) throws Exception {
    ServletUtil.CALC_EXECUTOR.shutdown();
    ServletUtil.TASK_EXECUTOR.shutdown();
    ServletUtil.CALC_EXECUTOR =
        MoreExecutors.listeningDecorator(Executors.newFixedThreadPool(ServletUtil.THREAD_COUNT));
    ServletUtil.TASK_EXECUTOR = Executors.newSingleThreadExecutor();
  }

  private static AwsProxyRequest getAwsProxyRequest(String path) {

    final AwsProxyRequest awsProxyRequest = new AwsProxyRequest();
    awsProxyRequest.setHttpMethod("GET");
    awsProxyRequest.setPath(path);
    return awsProxyRequest;
  }

}
